// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2020 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package cmd

import (
	"archive/zip"
	"code.vikunja.io/api/pkg/db"
	"code.vikunja.io/api/pkg/files"
	"code.vikunja.io/api/pkg/log"
	"code.vikunja.io/api/pkg/version"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
	"io"
	"os"
	"time"
)

const compressionUsed = zip.Deflate

func init() {
	rootCmd.AddCommand(dumpCmd)
}

var dumpCmd = &cobra.Command{
	Use:   "dump",
	Short: "Dump all vikunja data into a zip file. Includes config, files and db.",
	PreRun: func(cmd *cobra.Command, args []string) {
		fullInit()
	},
	Run: func(cmd *cobra.Command, args []string) {
		filename := "vikunja-dump_" + time.Now().Format("2006-01-02_15-03-05") + ".zip"
		dumpFile, err := os.Create(filename)
		if err != nil {
			log.Criticalf("Error opening dump file: %s", err)
		}
		defer dumpFile.Close()

		dumpWriter := zip.NewWriter(dumpFile)
		defer dumpWriter.Close()

		// Config
		err = writeFileToZip(viper.ConfigFileUsed(), dumpWriter)
		if err != nil {
			log.Criticalf("Error saving config file: %s", err)
		}
		log.Info("Dumped config file")

		// Version
		err = writeBytesToZip("VERSION", []byte(version.Version), dumpWriter)
		if err != nil {
			log.Criticalf("Error saving version: %s", err)
		}
		log.Info("Dumped version")

		// Database
		data, err := db.Dump()
		if err != nil {
			log.Criticalf("Error saving database data: %s", err)
		}
		for t, d := range data {
			err = writeBytesToZip("database/"+t+".json", d, dumpWriter)
			if err != nil {
				log.Criticalf("Error writing database table %s: %s", t, err)
			}
		}
		log.Info("Dumped database")

		// Files
		files.Dump()
	},
}

func writeFileToZip(filename string, writer *zip.Writer) error {
	fileToZip, err := os.Open(filename)
	if err != nil {
		return err
	}
	defer fileToZip.Close()

	// Get the file information
	info, err := fileToZip.Stat()
	if err != nil {
		return err
	}

	header, err := zip.FileInfoHeader(info)
	if err != nil {
		return err
	}

	header.Name = info.Name()

	// Change to deflate to gain better compression
	// see http://golang.org/pkg/archive/zip/#pkg-constants
	header.Method = compressionUsed

	w, err := writer.CreateHeader(header)
	if err != nil {
		return err
	}
	_, err = io.Copy(w, fileToZip)
	return err
}

func writeBytesToZip(filename string, data []byte, writer *zip.Writer) (err error) {
	header := &zip.FileHeader{
		Name:   filename,
		Method: compressionUsed,
	}
	w, err := writer.CreateHeader(header)
	if err != nil {
		return err
	}
	_, err = w.Write(data)
	return
}
